import { Subscription } from 'rxjs';

import {
  Component,
  ViewChild,
  OnDestroy,
  OnInit,
  Type,
  Injector
} from '@angular/core';
import { Router, NavigationEnd } from '@angular/router';

import { MatTabChangeEvent, MatTabGroup } from '@angular/material/tabs';
import { MatSidenav } from '@angular/material/sidenav';

import { Store, select } from '@ngrx/store';

import { LogService } from '@ucap/ng-logger';

import { AppSelector } from '@app/store/state';
import { UCAP_PATH_PARAM } from '@app/types/tokens';

const NAVS = ['/group', '/chat', '/organization', '/message'];

@Component({
  selector: 'app-layouts-default',
  templateUrl: './default.layout.component.html',
  styleUrls: ['./default.layout.component.scss']
})
export class DefaultLayoutComponent implements OnInit, OnDestroy {
  @ViewChild('navTabGroup', { static: true })
  navTabGroup: MatTabGroup;

  @ViewChild('leftSidenav', { static: true })
  leftSidenav: MatSidenav;

  sidenavComponent: Type<any>;
  sidenavInjector: Injector;
  contentComponent: Type<any>;
  contentInjector: Injector;

  private windowSizeSubscription: Subscription;
  private routerEventSubscription: Subscription;

  constructor(
    private router: Router,
    private injector: Injector,
    private store: Store<any>,
    private logService: LogService
  ) {
    this.routerEventSubscription = this.router.events.subscribe((event) => {
      if (event instanceof NavigationEnd) {
        this.onRoute(event.url);
      }
    });
  }

  ngOnInit(): void {
    this.windowSizeSubscription = this.store
      .pipe(select(AppSelector.windowSize))
      .subscribe((size) => {
        if (size.width < 780) {
          if (this.leftSidenav.opened) {
            this.leftSidenav.close();
          }
        } else {
          if (!this.leftSidenav.opened) {
            this.leftSidenav.open();
          }
        }
      });

    this.onRoute(this.router.url);
  }

  ngOnDestroy(): void {
    if (!this.windowSizeSubscription) {
      this.windowSizeSubscription.unsubscribe();
    }
    if (!!this.routerEventSubscription) {
      this.routerEventSubscription.unsubscribe();
    }
  }

  onSelectedTabChange(event: MatTabChangeEvent) {
    this.router.navigate([NAVS[event.index]]);
  }

  onRoute(url: string) {
    this.navTabGroup.selectedIndex = NAVS.findIndex((v) => url.startsWith(v));

    this.sidenavInjector = Injector.create({
      providers: [{ provide: UCAP_PATH_PARAM, useValue: 'sidenav' }],
      parent: this.injector
    });
    this.contentInjector = Injector.create({
      providers: [{ provide: UCAP_PATH_PARAM, useValue: 'content' }],
      parent: this.injector
    });

    switch (url) {
      case '/organization':
        import('@app/pages/organization/organization.page.module').then((m) => {
          this.sidenavComponent = m.SidenavPageComponent;
          this.contentComponent = m.IndexPageComponent;
        });
        break;
      case '/group':
        import('@app/pages/group/group.page.module').then((m) => {
          this.sidenavComponent = m.SidenavPageComponent;
          this.contentComponent = m.IndexPageComponent;
        });
        break;
      case '/chat':
        import('@app/pages/chat/chat.page.module').then((m) => {
          this.sidenavComponent = m.SidenavPageComponent;
          this.contentComponent = m.IndexPageComponent;
        });
        break;
      case '/message':
        import('@app/pages/message/message.page.module').then((m) => {
          this.sidenavComponent = m.SidenavPageComponent;
          this.contentComponent = m.IndexPageComponent;
        });
        break;

      default:
        break;
    }
  }

  onClickToggleLeftSidenav() {
    if (this.leftSidenav.opened) {
      this.leftSidenav.close();
    } else {
      this.leftSidenav.open();
    }
  }
}
